﻿#pragma TextEncoding	= "UTF-8"
#pragma rtGlobals 		= 3
#pragma IgorVersion		= 6.36
#pragma ModuleName		= SpectraGraphTools
#pragma version			= 1.0
#pragma DefaultTab		= {3,20,4}																// Set default tab width in Igor Pro 9 and later

// --------------------- Project Updater header ----------------------
// If you're using Igor Pro 8 or later and have Project Updater
// installed, this package can check periodically for new releases.
// https://www.wavemetrics.com/project/Updater
static Constant kProjectID = 21562																// the project node on IgorExchange
static StrConstant ksShortTitle = "Spectra Graph Tools"											// the project short title on IgorExchange

//________________________________________________________________________________________________
//	Written by Stephan Thuermer - https://www.wavemetrics.com/user/chozo
//
//	This is a collection of functions for modifying traces and graphs.
//	Contents:
//		- Basic Graph Style and Label Macros
//		- Quick Normalization :		Normalize to unit area or height / scale the traces in a graph
//		- SpectraWaterfall :		Generates a waterfall plot out of 2D data or a folder
//		- BuildTraceOffsetPanel :	A small panel to offset and color traces
//		- BuildGraphSizePanel :		A small panel to set the graph size and margins
//
// 2021-03-01 - ver. 1.00: Initial public release
//________________________________________________________________________________________________

Static Constant kOffsetIncrStep = 0.005															// fraction of the total range to increase / decrease values each step

//-------------------------------- functions for user code ---------------------------------------
Static Function TraceTool_FetchXShift(input, unit)												// extract predetermined X offset shift values from data
	Wave input
	String &unit																				// optional unit string
	Variable xShift = NaN																		// the shift value which will be preset in the panel
	// ++++++++++++++++++++++ get the photon energy as shift value +++++++++++++++++++++++++++++++
	Variable neg = 1
	String notes = note(input)
	String energy = StringByKey("Xray energy", notes, "=","\r")									// find the photon energy
	if (!strlen(energy))
		energy = StringByKey("photon energy ", notes, "=","\r")									// try a different key
	endif
	if (!strlen(energy))
		energy = StringByKey("Excitation Energy", notes, "=","\r")								// try yet a different key
		neg = -1																				// this data requires flipping
	endif
	unit = "eV"
	xShift = str2num(energy)*neg
	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	return xShift 																				// return NaN for no change and return negative values for flipping the trace
End

Static Function TraceTool_FetchYShift(input,unit)												// extract predetermined Y offset shift values
	Wave input
	String &unit
	return NaN	 																				// return NaN for no change and return negative values for flipping the trace
End
//------------------------------------------------------------------------------------------------

Menu "Spectra Tools"
	"-"
	"Normalize Traces to 1",/Q, ScaleTo1()
	"Normalize Traces to 1 at Cursor A",/Q, ScaleTo1(xpos=hcsr(A))
	"Normalize Traces to Area",/Q, ScaleTo1(toarea=1)
	"-"
	"Waterfall Plot of 2D Data or Folder", /Q, CreateBrowser; SpectraWaterfall()
	"Graph Size Panel ...", /Q, BuildGraphSizePanel()
	"Trace Offset Panel ...", /Q, BuildTraceOffsetPanel()
End

Menu "GraphPopup"
	"Left: Autoscale Only Visible", /Q,	SetAxis/A=2/E=0 left
	"Left: Autoscale From Zero", /Q, SetAxis/A=2/E=1 left
	"Traces: Undo All Scaling", /Q, ModifyGraph muloffset={0,0}
	"Traces: Undo All Offsets", /Q, ModifyGraph offset={0,0}
End

//________________________________________________________________________________________________
//
//										Standard Graph Macros
//________________________________________________________________________________________________

Proc BoxStyle() : GraphStyle
	PauseUpdate; Silent 1
	GetAxis/Q right
	if (V_flag == 1)
		ModifyGraph/Z mirror(left)=2
	endif
	GetAxis/Q top
	if (V_flag == 1)
		ModifyGraph/Z mirror(bottom)=2
	endif
	ModifyGraph noLabel=0
	ModifyGraph standoff=0
	ModifyGraph ZisZ=1
	ModifyGraph tick=0
	ModifyGraph notation(left)=1
	ModifyGraph axisOnTop=1
	ModifyGraph minor=1
	if ((GetKeyState(0) & 2^0) != 0)															// ctrl / command = paper style (no left axis notation)
		ModifyGraph tick(left)=3
		ModifyGraph noLabel(left)=1
	endif
	ResumeUpdate
EndMacro

Proc XAS_Label() : GraphStyle
	Label/Z left "\u#2Intensity (arb.u.) \\E"
	Label/Z bottom "Photon energy (\U)"
EndMacro

Proc XES_Label() : GraphStyle
	Label/Z left "\u#2Intensity (arb.u.) \\E"
	Label/Z bottom "Emission Energy (\U)"
EndMacro

Proc TOF_Label() : GraphStyle
	Label/Z left "\u#2Intensity (arb.u.) \\E"
	Label/Z bottom "Flight time (\U)"
EndMacro

Proc PKE_Label() : GraphStyle
	Label/Z left "\u#2Intensity (arb.u.) \\E"
	Label/Z bottom "\u#2"+SelectString((GetKeyState(0) & 2^0) != 0, "Kinetic", "Binding")+" Energy (eV)"	// BE label with ctrl / command
EndMacro

//________________________________________________________________________________________________
//
//							Scaling Normalization of Graph Traces
//________________________________________________________________________________________________

Function ScaleTo1([smth,xpos,toarea])
	Variable smth, xpos, toarea
	String traces  = TraceNameList("",";",1)
	Variable items = ItemsInList(Traces)
	if (items == 0)
		return 0
	endif

	Variable i, shiftBaseline = !(GetKeyState(0) == 4)											// don't shift background if shift is pressed
	for (i = 0; i < items; i += 1)
		String curr	= StringFromList(i,traces)
		Wave inwave	= TraceNameToWaveRef("", curr)
		if (WaveDims(inwave) == 1)
			Duplicate/FREE inwave work
		elseif (WaveDims(inwave) == 2)
			String range = StringByKey("YRANGE", TraceInfo("",curr,0))
			Variable col = strsearch(range,"[",Inf,1)
			range = range[col+1,inf]
			col = str2num(RemoveEnding(range,"]"))
			if (numtype(col) == 0)
				Duplicate/R=[][col]/FREE inwave work
			else
				continue
			endif
		else
			return 0
		endif
		if( !ParamIsDefault(smth) && smth > 0)													// optional smoothing before scaling
			Smooth smth, work
		endif
		
		Variable minval	= wavemin(work)*shiftBaseline
		Variable maxval	= wavemax(work)
		if (!ParamIsDefault(xpos) && numtype(xpos) == 0)
			Variable pntpos = x2pnt(work, xpos)
			if (pntpos > 0 && pntpos < DimSize(work,0)-1)										// make sure to stay within range
				maxval = work(xpos)
			endif
		endif
		Variable scale = maxval-minval
		if (!ParamIsDefault(toarea) && toarea == 1)
			scale  = area(work)
			minval = 0
		endif
		scale = scale == 0 ? 1 : scale
		ModifyGraph offset($curr)={,-minval/scale},muloffset($curr)={,1/scale}
	endfor
End

//________________________________________________________________________________________________
//
//					Creates Waterfall Plot from a 2D Wave or a Folder of Waves
//________________________________________________________________________________________________

Function SpectraWaterfall()
	Wave/Z inwave = $GetBrowserSelection(0)														// get the first selected wave
	if (!WaveExists(inwave))
		Print "Nothing selected"
		return -1
	endif
	String list	= ""
	Variable OneD = WaveDims(inwave) == 1														// first check, if the incoming wave is 1D or 2D
	DFREF saveDFR = GetDataFolderDFR()
	if (OneD)
		SetDataFolder GetWavesDataFolderDFR(inwave)
		list = WaveList("!Int_*", ";", "DIMS:1")
		list = SortList(list, ";", 16)
	endif
	
	Display/W=(230,50,650,450)/K=1/N=$UniqueName("WaterfallGraph", 6, 0) as "Waterfall of "+NameOfWave(inwave)
	Variable i, items = OneD ? ItemsInList(list) : DimSize(inwave,1)
	for (i = items-1; i > -1; i -=1)															// go backwards (in reverse order, so that the later traces are BEHIND the earlier ones)
		if (OneD)
			AppendToGraph/Q $Stringfromlist(i,list)
		else
			AppendToGraph/Q inwave[][i]
		endif
	endfor
	SetDataFolder saveDFR
	
	Execute/Q "BoxStyle()"
	ColorSpectrumFromTable(0, "Rainbow")
	
	list = TraceNameList("",";",1)
	Variable initOffset	= WaveMax(inwave) * 0.05												// the initial distance will be set in percent from the maximal value of the selected wave
	for (i = 1; i < items; i += 1)
		ModifyGraph offset($StringFromList(items-1 - i,list))={,i*initOffset}
	endfor
	return 0
End

//________________________________________________________________________________________________
//
//										TRACE OFFSET PANEL
//________________________________________________________________________________________________

Function BuildTraceOffsetPanel()
	if (WinType("OffsetPanel"))
		DoWindow/F OffsetPanel
		return 0
	endif
	NewPanel/K=1/W=(450,60,885,470)/N=OffsetPanel as "Modify Traces"
	ModifyPanel /W=OffsetPanel ,fixedSize= 1 ,noEdit=1
	SetWindow OffsetPanel hook(UpdateHook)=OffsetPanelUpdateFunc

	SetDrawLayer UserBack
	DrawLine 15,275,205,275
	DrawLine 15,340,205,340
	DrawLine 230,230,420,230
	DrawLine 230,312,420,312
	// +++++++++++++++++++++++++++++++++++++++ left side +++++++++++++++++++++++++++++++++++++++++
	GroupBox MultiTraceBox		,pos={6,7}		,size={208,367}	,title="Multiple Traces"		,fstyle=1		,fColor=(0,16000,65535)
	CheckBox cHiddenTraces		,pos={28,28}	,size={60,22}	,title="Include Hidden Traces"	,value = 1		,help={"Include hidden traces when scaling."}
	SetVariable vaSetYOffset	,pos={15,65}	,size={90,16}	,title="y:"
	SetVariable vaScaleYOffset	,pos={15,105}	,size={90,16}	,title="y:"
	SetVariable vaShiftYOffset	,pos={15,145}	,size={90,16}	,title="y:"
	SetVariable vaSetXOffset	,pos={115,65}	,size={90,16}	,title="x:"
	SetVariable vaScaleXOffset	,pos={115,105}	,size={90,16}	,title="x:"
	SetVariable vaShiftXOffset	,pos={115,145}	,size={90,16}	,title="x:"
	
	TitleBox SetTitle			,pos={15,48}	,size={160,13}	,title="Set Offset per Trace"
	TitleBox ScaleTitle			,pos={15,88}	,size={160,13}	,title="Scale Offset per Trace"
	TitleBox ShiftTitle			,pos={15,128}	,size={160,13}	,title="Shift Offset of All Traces"
	Button ResetYButton			,pos={15,170}	,size={90,22}	,title="Y Reset"								,help={"Resets all Y offsets to 0."}
	Button ResetXButton			,pos={115,170}	,size={90,22}	,title="X Reset"								,help={"Resets all X offsets to 0."}
	Button ReverseButton		,pos={15,200}	,size={190,22}	,title="Reverse Sorting"						,help={"Reverses the order in which the traces are sorted in the graph."}
	SetVariable TraceFilter		,pos={15,230}	,size={190,16}	,title="Filter:"								,help={"Apply offsets only to traces with matching name. You can use wildcard characters."}
	CheckBox cOmitFilter		,pos={48,252}	,size={90,22}	,title="Filter Excludes Traces"	,value = 0		,help={"The filter keyword is used to omit traces instead of including them."}

	PopupMenu ColorSelect		,pos={15,285}	,size={190,22}									,bodyWidth=190
	Button ColorButton			,pos={15,310}	,size={110,22}	,title="Colorize"								,help={"Colorize traces with currently selected color spectrum. To reverse spectrum hold 'ctrl'/'command'."}
	CheckBox cReverseColor		,pos={135,313}	,size={60,22}	,title="in Reverse"				,value = 0		,help={"Applies the color in reverse (also works directly by holding 'ctrl'/'command' while pressing the button.)"}
	PopUpMenu ProfileSelect		,pos={15,347}	,size={190,20}	,title="Profile"				,bodywidth=152
	// +++++++++++++++++++++++++++++++++++++++ right side ++++++++++++++++++++++++++++++++++++++++
	GroupBox SingleTraceBox		,pos={220,7}	,size={208,367}	,title="Individual Traces"		,fstyle=1		,fColor=(0,16000,65535)
	TitleBox CsrTitle			,pos={230,28}	,size={60,23}	,title="Cursor:"				,fstyle=1
	CheckBox traceCsrA			,pos={285,28}	,size={60,22}	,title="A"
	CheckBox traceCsrB			,pos={320,28}	,size={60,22}	,title="B"
	CheckBox traceCsrC			,pos={355,28}	,size={60,22}	,title="C"
	CheckBox traceCsrD			,pos={390,28}	,size={60,22}	,title="D"
	
	TitleBox trSetTitle			,pos={230,48}	,size={170,13}	,title="Set Trace Offset"
	TitleBox trScaleTitle		,pos={230,88}	,size={170,13}	,title="Set Trace Multiplier"
	TitleBox trShiftTitle		,pos={230,128}	,size={170,13}	,title="Reposition at Cursor"
	SetVariable trSetYOffset	,pos={230,65}	,size={90,16}	,title="y:"
	SetVariable trScaleYOffset	,pos={230,105}	,size={90,16}	,title="y:"
	SetVariable trShiftYOffset	,pos={230,145}	,size={90,16}	,title="y:"
	SetVariable trSetXOffset	,pos={330,65}	,size={90,16}	,title="x:"
	SetVariable trScaleXOffset	,pos={330,105}	,size={90,16}	,title="x:"
	SetVariable trShiftXOffset	,pos={330,145}	,size={90,16}	,title="x:"
	SetVariable trScaleNorm		,pos={230,173}	,size={140,16}	,title="Normalize:"				,bodyWidth=80	,help={"Adjusts the multiplier to normalize the trace height to this value at the current cursor's position (excluding the Y offset)."}
	Button trNormTo1Button		,pos={380,172}	,size={40,20}	,title="to 1"									,help={"Normalizes trace height to 1 at current cursor position (excluding the Y offset)."}
	Button trResetYButton		,pos={230,200}	,size={90,22}	,title="Y Reset"								,help={"Resets Y offset and multiplier to 0."}
	Button trResetXButton		,pos={330,200}	,size={90,22}	,title="X Reset"								,help={"Resets X offset and multiplier to 0."}
	
	TitleBox trCsrControlTitle	,pos={230,235}	,size={190,13}	,title="Control Cursor Position"
	Button trPrevTraceButton	,pos={230,253}	,size={90,22}	,title="< Prev Trace"							,help={"Moves the cursor to onto the previous trace in the list of traces."}
	Button trNextTraceButton	,pos={330,253}	,size={90,22}	,title="Next Trace >"							,help={"Moves the cursor to onto the next trace in the list of traces."}
	Button trGoToMaxButton		,pos={230,283}	,size={90,22}	,title="Go to Max"								,help={"Sets the cursor to trace's maximum."}
	Button trGoToMinButton		,pos={330,283}	,size={90,22}	,title="Go to Min"								,help={"Sets the cursor to trace's minimum."}
	
	SetVariable trOffxDelta		,pos={275,320}	,size={100,16}	,title="dx:"					,bodyWidth=80	,help={"Negative values will flip trace in X direction."}
	SetVariable trOffyDelta		,pos={275,348}	,size={100,16}	,title="dy:"					,bodyWidth=80	,help={"Negative values will flip trace in Y direction."}
	Button trAddXButton			,pos={385,318}	,size={35,20}	,title="+"			,fstyle=1	,fsize=14		,help={"Add X delta value."}
	Button trRemXButton			,pos={230,318}	,size={35,20}	,title="-"			,fstyle=1	,fsize=14		,help={"Subtract X delta value."}
	Button trAddYButton			,pos={385,346}	,size={35,20}	,title="+"			,fstyle=1	,fsize=14		,help={"Add Y delta value."}
	Button trRemYButton			,pos={230,346}	,size={35,20}	,title="-"			,fstyle=1	,fsize=14		,help={"Subtract Y delta value."}
	
	Button vaApplyAllButton		,pos={10,380}	,size={200,22}	,title="Apply All to Data (overwrite!)"			,help={"Applies all offsets and scales to 1D data. Will overwrite the original waves, so use this with care!"}
	Button trApplyCurButton		,pos={225,380}	,size={200,22}	,title="Apply to This Data (overwrite!)"		,help={"Applies offsets and scales to the currently selected 1D data. Will overwrite the original waves, so use this with care!"}
	// ++++++++++++++++++++++++++++++++++++++++ settings +++++++++++++++++++++++++++++++++++++++++
	SetVariable vaSetYOffset	,value=_NUM:0	,limits={0,inf,1}
	SetVariable vaScaleYOffset	,value=_NUM:1	,limits={0,inf,0.1}		,userdata="1"
	SetVariable vaShiftYOffset	,value=_NUM:0	,limits={-inf,inf,1}
	SetVariable vaSetXOffset	,value=_NUM:0	,limits={-inf,inf,0.1}
	SetVariable vaScaleXOffset	,value=_NUM:1	,limits={0,inf,0.1}		,userdata="1"
	SetVariable vaShiftXOffset	,value=_NUM:0	,limits={-inf,inf,1}
	
	SetVariable trSetYOffset	,value=_NUM:0	,limits={-inf,inf,1}
	SetVariable trScaleYOffset	,value=_NUM:1	,limits={-inf,inf,0.1}
	SetVariable trShiftYOffset	,value=_NUM:0	,limits={-inf,inf,0}
	SetVariable trSetXOffset	,value=_NUM:0	,limits={-inf,inf,0.1}
	SetVariable trScaleXOffset	,value=_NUM:1	,limits={-inf,inf,0.1}
	SetVariable trShiftXOffset	,value=_NUM:0	,limits={-inf,inf,0}
	SetVariable trScaleNorm		,value=_NUM:0	,limits={-inf,inf,1}
	
	SetVariable trOffxDelta		,value=_NUM:0	,limits={-inf,inf,0}
	SetVariable trOffyDelta		,value=_NUM:0	,limits={-inf,inf,0}
	SetVariable TraceFilter		,value=_STR:""
	
	PopupMenu ColorSelect		,mode=2	,value="*COLORTABLEPOPNONAMES*"
	PopupMenu ProfileSelect		,mode=1	,value=TraceTool_ProfileWaveList()	,proc=TraceTool_ProfileSelect
	
	ModifyControlList ControlNameList("OffsetPanel",";","tr*")			,disable=2				// disable all cursor related controls for now
	ModifyControlList ControlNameList("OffsetPanel",";","traceCsr*")	,proc=TraceTool_CursorSelector	,mode = 1
	ModifyControlList ControlNameList("OffsetPanel",";","*Button")		,proc=TraceTool_ButtonFunctions
	ModifyControlList ControlNameList("OffsetPanel",";","vaS*") 		,proc=TraceTool_MultiTraceVars	,format="%g"
	ModifyControlList ControlNameList("OffsetPanel",";","trS*") 		,proc=TraceTool_SingleTraceVars	,format="%g"
	ModifyControlList ControlNameList("OffsetPanel",";","*Title")+"MultiTraceBox;SingleTraceBox;"		,frame=0
#if IgorVersion() >= 7.00
	ModifyControlList ControlNameList("OffsetPanel",";","vaS*")+"ColorSelect;ProfileSelect;" 			,focusRing=0
#endif
	if (CmpStr(IgorInfo(2), "Macintosh") == 0)
		ModifyControlList ControlNameList("OffsetPanel",";","*") ,fsize=10
	endif
	FetchLatestTraceValues()																	// set variable values from top window
	return 0
End
//+++++++++++++++++++++++++++++++++++++ popup help functions +++++++++++++++++++++++++++++++++++++
Function/S TraceTool_ProfileWaveList()
	return "none;"+WaveList("*",";", "DIMS:1")
End

//################################ panel update helper function ##################################

Function OffsetPanelUpdateFunc(s)
	STRUCT WMWinHookStruct &s
	If (s.eventCode == 0)	// Activate
		FetchLatestTraceValues()
		return 1
	endif
	return 0
End

//################################################################################################

Function TraceTool_CursorSelector(s) : CheckBoxControl
	STRUCT WMCheckboxAction &s
	if(s.eventCode == 2)
		String ctrl = RemoveFromList(s.ctrlName, ControlNameList("OffsetPanel",";","traceCsr*"))
		Variable i
		for (i = 0; i < ItemsInList(ctrl); i += 1)
			CheckBox $StringFromList(i,ctrl) ,win=OffsetPanel ,value=0
		endfor
		FetchLatestCursorValues(ReplaceString("traceCsr", s.ctrlName, ""))
	endif
	return 0
End

//################################################################################################

Static Function FetchLatestTraceValues()														// update both left and right side values (if cursors are set)
	String gName = WinName(0,1)
	if (!strlen(gName))
		return 0
	endif
	Variable xRange, yRange
	String xUnit = "", yUnit = ""
	Variable error = GetAxisRange(xRange, yRange, xUnit, yUnit, gName)
	if (error)
		return -1
	endif
	
	String traces = ""
	Variable items = FetchTraces(gName,traces,0)
	if (items > 1)																				// calculate shift and offset from first two traces
		Variable xOff = 0, yOff = 0, xShift = 0,  yShift = 0
		sscanf StringByKey("offset(x)", TraceInfo(gName,StringFromList(items-2,traces),0), "="), "{%f,%f}", xOff, yOff
		sscanf StringByKey("offset(x)", TraceInfo(gName,StringFromList(items-1,traces),0), "="), "{%f,%f}", xShift, yShift	
	endif
	xUnit = SelectString(strlen(xUnit),""," "+xUnit)
	yUnit = SelectString(strlen(yUnit),""," "+yUnit)
	
	DoWindow /T OffsetPanel, "Modify Traces: "+WinName(0,1)
	Wave data = TraceNameToWaveRef(gName, StringFromList(0,traces))												
	Button vaApplyAllButton 	,win=OffsetPanel 	,disable=2*(WaveDims(data) != 1)			// this button only works for 1D traces
	SetVariable vaSetYOffset	,win=OffsetPanel	,value=_NUM:yOff-yShift	,format="%g"+yUnit	,limits={-inf,inf,yRange*kOffsetIncrStep}
	SetVariable vaShiftYOffset	,win=OffsetPanel	,value=_NUM:yShift		,format="%g"+yUnit	,limits={-inf,inf,yRange*kOffsetIncrStep}
	SetVariable vaSetXOffset	,win=OffsetPanel	,value=_NUM:xOff-xShift	,format="%g"+xUnit	,limits={-inf,inf,xRange*kOffsetIncrStep}
	SetVariable vaShiftXOffset	,win=OffsetPanel	,value=_NUM:xShift		,format="%g"+xUnit	,limits={-inf,inf,xRange*kOffsetIncrStep*10}
	
	PopupMenu ProfileSelect 	,win=OffsetPanel 	,mode=1										// update the selected profile or select none
	if (FindListItem("Profileleft", AxisList("")) >= 0)
		Wave profile = TraceNameToWaveRef("", "AddTraceProfile")
		PopupMenu ProfileSelect	,win=OffsetPanel	,popmatch=nameofwave(profile)
	endif
	ModifyControlList ControlNameList("OffsetPanel",";","tr*") ,disable=2						// disable cursor related controls for now
	// if (WaveDims(data) != 1)																	// no use for images
		// return 0
	// endif
	//------------------------------------- cursor controls --------------------------------------
	String csr = "A;B;C;D;", currCsr															// fetch available cursor list
	Variable i, selCsr = -1,  setCsr = -1, csrActive = 0
	for (i = 0; i < ItemsInList(csr); i += 1)													// find currently selected cursor
		currCsr = "traceCsr"+StringFromList(i,csr)
		ControlInfo/W=OffsetPanel $currCsr
		if (V_Value)
			CheckBox $currCsr ,win=OffsetPanel ,value=0											// disable for now
			selCsr = i
			break
		endif
	endfor

	for (i = 0; i < ItemsInList(csr); i += 1)
		currCsr = StringFromList(i,csr)
		if (strlen(CsrInfo($currCsr,gName)))
			if (csrActive == 0 && selCsr == i)													// the currently selected cursor is active and can stay
				csrActive = 1
				setCsr = i
			endif
			if (setCsr == -1)																	// find the first free cursor from left
				setCsr = i
			endif
			CheckBox $("traceCsr"+currCsr) ,win=OffsetPanel ,disable=0
		endif
	endfor
	if (setCsr == -1)
		SetWindow OffsetPanel, userdata(cursor)=""												// clear cursor user data
		return 0
	endif
	//---------------------------------- if cursor was found -------------------------------------
	currCsr = StringFromList(setCsr,csr)
	CheckBox $("traceCsr"+currCsr) ,win=OffsetPanel ,value=1
	String ctrlList = RemoveFromList(ControlNameList("OffsetPanel",";","traceCsr*")+"vaApplyAllButton;",ControlNameList("OffsetPanel",";","tr*"))
	ModifyControlList ctrlList ,disable=0
	FetchLatestCursorValues(currCsr)															// now get cursor specific values (right side)
	return 0
End

//################################################################################################

Static Function FetchLatestCursorValues(csr)
	String csr
	String gName = WinName(0,1)
	if (!strlen(gName))
		return 0
	endif
	Variable xRange, yRange, shift
	String xUnit = "", yUnit = "", unit = ""
	Variable error = GetAxisRange(xRange, yRange, xUnit, yUnit, gName)
	if (error)
		return -1
	endif
	
	String trace = StringByKey("TNAME",CsrInfo($csr,gName))
	Variable xOff = 0, yOff = 0, xScale = 0, yScale = 0
	sscanf StringByKey("offset(x)", TraceInfo(gName,trace,0), "="), "{%f,%f}", xOff, yOff
	sscanf StringByKey("muloffset(x)", TraceInfo(gName,trace,0), "="), "{%f,%f}", xScale, yScale
	yScale = yScale == 0? 1 : yScale
	xScale = xScale == 0? 1 : xScale
	xUnit = SelectString(strlen(xUnit),""," "+xUnit)
	yUnit = SelectString(strlen(yUnit),""," "+yUnit)
	
	Variable csrShiftx = (hcsr($csr)*xScale+xOff)												// round to prevent floating point errors
	Variable csrShifty = (vcsr($csr)*yScale+yOff)
	csrShiftx = round(csrShiftx*10^6)/10^6
	csrShifty = round(csrShifty*10^6)/10^6
	
	Wave data = TraceNameToWaveRef(gName, trace)												
	Button trApplyCurButton 	,win=OffsetPanel 	,disable=2*(WaveDims(data) != 1)			// this button only works for 1D traces
	SetVariable trSetYOffset	,win=OffsetPanel	,value=_NUM:yOff				,format="%g"+yUnit	,limits={-inf,inf,yRange*kOffsetIncrStep}
	SetVariable trShiftYOffset	,win=OffsetPanel	,value=_NUM:csrShifty			,format="%g"+yUnit	,limits={-inf,inf,yRange*kOffsetIncrStep*10}
	SetVariable trScaleYOffset	,win=OffsetPanel	,value=_NUM:yScale				,format="%g"
	SetVariable trSetXOffset	,win=OffsetPanel	,value=_NUM:xOff				,format="%g"+xUnit	,limits={-inf,inf,xRange*kOffsetIncrStep}
	SetVariable trShiftXOffset	,win=OffsetPanel	,value=_NUM:csrShiftx			,format="%g"+xUnit	,limits={-inf,inf,xRange*kOffsetIncrStep*10}
	SetVariable trScaleXOffset	,win=OffsetPanel	,value=_NUM:xScale				,format="%g"
	SetVariable trScaleNorm		,win=OffsetPanel	,value=_NUM:vcsr($csr)*yScale	,format="%g"+yUnit
	
	SetVariable trOffxDelta		,win=OffsetPanel	,format="%g"								// first reset unit
	SetVariable trOffyDelta		,win=OffsetPanel	,format="%g"
	shift = TraceTool_FetchXShift(TraceNameToWaveRef(gName, trace),unit)
	if (numtype(shift) == 0)
		SetVariable trOffxDelta	,win=OffsetPanel	,value=_NUM:shift	,format="%g"+SelectString(strlen(unit),""," "+unit)
	endif
	unit = ""
	shift = TraceTool_FetchYShift(TraceNameToWaveRef(gName, trace),unit)
	if (numtype(shift) == 0)
		SetVariable trOffyDelta	,win=OffsetPanel	,value=_NUM:shift	,format="%g"+SelectString(strlen(unit),""," "+unit)
	endif
	
	SetWindow OffsetPanel, userdata(trace)=trace												// save into panel's user data
	SetWindow OffsetPanel, userdata(cursor)=csr
	return 0
End

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

Static Function GetAxisRange(xRange, yRange, xUnit, yUnit, gName)
	Variable &xRange, &yRange
	String &xUnit, &yUnit, gName
	String Axes = AxisList(gName), str
	if (WhichListItem("left",Axes) > -1)
		GetAxis/W=$gName/Q left
		yRange = abs(V_max - V_min)
		yUnit  = StringByKey("UNITS", AxisInfo("","left"))
	elseif (WhichListItem("right",Axes) > -1)
		GetAxis/W=$gName/Q right
		yRange = abs(V_max - V_min)
		yUnit  = StringByKey("UNITS", AxisInfo("","right"))
	else
		return -1
	endif
	if (WhichListItem("bottom",Axes) > -1)
		GetAxis/W=$gName/Q bottom
		xRange = abs(V_max - V_min)
		xUnit  = StringByKey("UNITS", AxisInfo("","bottom"))
	elseif (WhichListItem("top",Axes) > -1)
		GetAxis/W=$gName/Q top
		xRange = abs(V_max - V_min)
		xUnit  = StringByKey("UNITS", AxisInfo("","top"))
	else
		return -1
	endif
	sprintf str, "%.*g\r", 1, yRange;	yRange = str2num(str)									// rounding
	sprintf str, "%.*g\r", 1, xRange;	xRange = str2num(str)
	return 0
End

//################################## variable helper function ####################################

Function TraceTool_SingleTraceVars(SV) : SetVariableControl
	STRUCT WMSetVariableAction &SV
	if (SV.eventCode == 1 || SV.eventCode == 2)
		SV.blockReentry=1
		String gName = WinName(0,1)
		if (!strlen(gName))
			return 0
		endif
		
		String trace = GetUserData("OffsetPanel", "", "trace")									// get current trace name and cursor from panel user data
		String csr   = GetUserData("OffsetPanel", "", "cursor")
		if (strlen(trace) == 0 || strlen(csr) == 0)
			return 0
		endif
		
		Variable xScale = 0,  yScale = 0
		sscanf StringByKey("muloffset(x)", TraceInfo(gName,trace,0), "="), "{%f,%f}", xScale, yScale
		yScale = yScale == 0? 1 : yScale
		xScale = xScale == 0? 1 : xScale
		if (StringMatch(SV.ctrlName,"trScale*"))
			SV.dval = abs(SV.dval) < 10e-6 ? sign(SV.dval)*10e-6 : SV.dval						// prevent input from getting very small
		endif
		
		StrSwitch(SV.ctrlName)
			case "trSetYOffset":
				ModifyGraph offset($trace)={,SV.dval}
			break
			case "trSetXOffset":
				ModifyGraph offset($trace)={SV.dval,}
			break
			case "trShiftYOffset":
				ModifyGraph offset($trace)={,(SV.dval-vcsr($csr)*yScale)}
			break
			case "trShiftXOffset":
				ModifyGraph offset($trace)={(SV.dval-hcsr($csr)*xScale),}
			break
			case "trScaleYOffset":
				ModifyGraph muloffset($trace)={,SV.dval}
			break
			case "trScaleXOffset":
				ModifyGraph muloffset($trace)={SV.dval,}
			break
			case "trScaleNorm":
				if (vcsr($csr) != 0)
					ModifyGraph muloffset($trace)={,SV.dval/vcsr($csr)}
				endif
			break
		EndSwitch	
		FetchLatestCursorValues(csr)
		AdjustProfileAxisScaling()
	endif
	return 0
End

//################################################################################################

Function TraceTool_MultiTraceVars(SV) : SetVariableControl
	STRUCT WMSetVariableAction &SV
	if (SV.eventCode == 1 || SV.eventCode == 2)
		SV.blockReentry=1
		String gName = WinName(0,1)
		if (!strlen(gName))
			return 0
		endif
		String traces = ""
		Variable items = FetchTraces(gName,traces,0)
		if (items == 0)
			return 0
		endif
		
		Variable curXOff = 0, curYOff = 0, xOff = 0, yOff = 0, YnotX = 0, i
		sscanf StringByKey("offset(x)", TraceInfo(gName,StringFromList(items-1,traces),0), "="), "{%f,%f}", xOff, yOff
		StrSwitch(SV.ctrlName)
			case "vaSetYOffset":
				YnotX = 1
			case "vaSetXOffset":
				for (i = 0; i < items; i += 1)
					if (YnotX)
						ModifyGraph offset($StringFromList(items-1 - i,traces))={,i*SV.dval+yOff}
					else
						ModifyGraph offset($StringFromList(items-1 - i,traces))={i*SV.dval+xOff,}
					endif
				endfor
			break
			case "vaScaleYOffset":
				YnotX = 1
			case "vaScaleXOffset":
				Variable PrevVal = str2num(SV.Userdata);	SV.Userdata = num2str(SV.dval)		// get previous multiplier from and write current value to control user data
				PrevVal = PrevVal != 0 ? PrevVal : 1											// make sure multiplier is not zero
				for (i = 0; i < items; i += 1)
					sscanf StringByKey("offset(x)", TraceInfo(gName,StringFromList(items-1 - i,traces),0), "="), "{%f,%f}", curXOff, curYOff
					if (YnotX)
						ModifyGraph offset($StringFromList(items-1 - i,traces))={,(curYOff-yOff)*SV.dval/PrevVal+yOff}
					else
						ModifyGraph offset($StringFromList(items-1 - i,traces))={(curXOff-xOff)*SV.dval/PrevVal+xOff,}
					endif
				endfor
			break
			case "vaShiftYOffset":
				YnotX = 1
			case "vaShiftXOffset":
				for (i = 0; i < items; i += 1)
					sscanf StringByKey("offset(x)", TraceInfo(gName,StringFromList(items-1 - i,traces),0), "="), "{%f,%f}", curXOff, curYOff
					if (YnotX)
						ModifyGraph offset($StringFromList(items-1 - i,traces))={,(curYOff-yOff)+SV.dval}
					else
						ModifyGraph offset($StringFromList(items-1 - i,traces))={(curXOff-xOff)+SV.dval,}
					endif
				endfor
			break
		EndSwitch
		String csr = GetUserData("OffsetPanel", "", "cursor")
		if (strlen(csr))
			FetchLatestCursorValues(csr)
		endif
		AdjustProfileAxisScaling()
	endif
	return 0
End

//++++++++++++++++++++++++++++++++ help function to get a trace list ++++++++++++++++++++++++++++++

Static Function FetchTraces(win,traces,getAll)
	String win, &traces
	Variable getAll
	
	Variable noHidden = 0, omit = 0
	String filter = ""
	if (WinType("OffsetPanel") && !getAll)																	// read settings from panel if open
		ControlInfo/W=OffsetPanel cHiddenTraces
		noHidden = !V_Value
		ControlInfo/W=OffsetPanel TraceFilter
		filter = S_Value
		ControlInfo/W=OffsetPanel cOmitFilter
		omit = V_Value
	endif
	
	traces = TraceNameList(win,";",1+4*noHidden)
	if (strlen(filter))
		if (omit)
			traces = RemoveFromList(ListMatch(traces,filter),traces)
		else
			traces = ListMatch(traces,filter)
		endif
	endif
	
	Variable i, items = ItemsInList(traces)
	String xAxis, remItem = ""
	for (i = 0; i < items; i += 1)																// remove possible vertical traces
		xAxis = StringByKey("XAXIS",  TraceInfo(win,StringFromList(items-1 - i,traces),0))
		if (!(StringMatch(xAxis,"bottom") || StringMatch(xAxis,"top")))
			remItem += StringFromList(items-1 - i,traces)+";"
		endif
	endfor
	traces = RemoveFromList(remItem,traces)
	items  = ItemsInList(traces)
	return items
End

//+++++++++++++++++++++++++++++++ help function for Profile Axis +++++++++++++++++++++++++++++++++

Static Function AdjustProfileAxisScaling()
	String gName = WinName(0,1)
	if (!strlen(gName) || WhichListItem("Profileleft", AxisList(gName)) < 0)
		return 0
	endif

	String traces = ""
	Variable items = FetchTraces(gName,traces,1)														// get all traces
	if (items == 0)
		return 0
	endif
	Variable xOff = 0, yOff = 0,currMax = 0, dataMin = 0, i, scale = 0
	for (i = 0; i < items; i += 1)
		sscanf StringByKey("offset(x)", TraceInfo(gName,StringFromList(i,traces),0), "="), "{%f,%f}", xOff, yOff
		if (yOff > currMax)
			currMax = yOff
			Wave work = TraceNameToWaveRef(gName,StringFromList(i,traces))
			dataMin = WaveMin(work)
		endif
	endfor
	dataMin = numtype(dataMin) != 0 ? 0 : dataMin

	DoUpdate; GetAxis/Q left
	scale = (currMax+dataMin-V_min)/(V_Max-V_min)												// calculate the ratio between highest offset and axis height
	scale = Limit(scale, 0.1, 1)																// prevent scaling to useless values
	ModifyGraph axisEnab(Profileleft)={0,scale}
	return 0
End

//################################### button helper function #####################################

Function TraceTool_ButtonFunctions(SB) : ButtonControl
	STRUCT WMButtonAction &SB
	if (SB.eventCode == 2)
		SB.blockReentry=1
		String gName = WinName(0,1)
		if (!strlen(gName))
			return 0
		endif
		
		Variable xOff = 0, yOff = 0, xScale = 0, yScale = 0, setX, setY
		String trace = GetUserData("OffsetPanel", "", "trace")									// get current selected trace name and cursor from panel user data
		String csr   = GetUserData("OffsetPanel", "", "cursor")
		if (strlen(trace) && strlen(csr))
			sscanf StringByKey("offset(x)", TraceInfo(gName,trace,0), "="), "{%f,%f}", xOff, yOff
			sscanf StringByKey("muloffset(x)", TraceInfo(gName,trace,0), "="), "{%f,%f}", xScale, yScale
		endif
		String traces = "",  list = ""
		Variable GoUp = 0, items, i
		StrSwitch(SB.ctrlName)
			case "ColorButton":
				ControlInfo/W=$(SB.win) ColorSelect
				String Table = S_value
				ControlInfo/W=$(SB.win) cReverseColor
				ColorSpectrumFromTable((V_Value || GetKeyState(0) & 1), table)
			break
			case "ReverseButton":
				items = FetchTraces(gName,traces,1)												// get all traces
				if (items == 0)
					break
				endif
				for (i = 0; i < items; i += 1)													// reverse trace list
					list += StringFromList(items-1 - i,traces) + ";"
				endfor
				sscanf StringByKey("offset(x)", TraceInfo(gName,StringFromList(items-1,traces),0), "="), "{%f,%f}", xOff, yOff
				ControlInfo/W=$(SB.win) vaSetYOffset;	setY = V_Value
				ControlInfo/W=$(SB.win) vaSetXOffset;	setX = V_Value
				String sortTraces = RemoveEnding(ReplaceString(";", list, ","), ",")
				Execute/Z "ReorderTraces _back_,{"+sortTraces+"}"								// execute reverse operation
				if (V_Flag)
					Print "Cannot reverse sorting: Too many traces."
					break
				else
					for (i = 0; i < items; i += 1)												// reapply offsets
						ModifyGraph offset($StringFromList(items-1 - i,traces))={i*setX+xOff,i*setY+yOff}
					endfor
					FetchLatestTraceValues()
				endif
			break
			case "ResetYButton":																// no break here... reverse button also resets trace offsets
				ModifyGraph offset={,0}//,muloffset={,0}
				SetVariable vaSetYOffset	,win=OffsetPanel	,value=_NUM:0
				SetVariable vaScaleYOffset	,win=OffsetPanel	,value=_NUM:1	,userdata="1"
				SetVariable vaShiftYOffset	,win=OffsetPanel	,value=_NUM:0
				AdjustProfileAxisScaling()														// resizes the profile axis
			break
			case "ResetXButton":
				ModifyGraph offset={0,}//,muloffset={0,}
				SetVariable vaSetXOffset	,win=OffsetPanel	,value=_NUM:0
				SetVariable vaScaleXOffset	,win=OffsetPanel	,value=_NUM:1	,userdata="1"
				SetVariable vaShiftXOffset	,win=OffsetPanel	,value=_NUM:0
				AdjustProfileAxisScaling()
			break
			case "trResetYButton":
				if (strlen(trace) && strlen(csr))
					ModifyGraph offset($trace)={,0} ,muloffset($trace)={,0}
					SetVariable trSetYOffset	,win=OffsetPanel	,value=_NUM:0
					SetVariable trScaleYOffset	,win=OffsetPanel	,value=_NUM:1
					SetVariable trShiftYOffset	,win=OffsetPanel	,value=_NUM:vcsr($csr)
					SetVariable trScaleNorm		,win=OffsetPanel	,value=_NUM:vcsr($csr)
					AdjustProfileAxisScaling()
				endif
			break
			case "trResetXButton":
				if (strlen(trace) && strlen(csr))
					ModifyGraph offset($trace)={0,} ,muloffset($trace)={0,}
					SetVariable trSetXOffset	,win=OffsetPanel	,value=_NUM:0
					SetVariable trScaleXOffset	,win=OffsetPanel	,value=_NUM:1
					SetVariable trShiftXOffset	,win=OffsetPanel	,value=_NUM:hcsr($csr)
					AdjustProfileAxisScaling()
				endif
			break
			case "trGoToMaxButton":
				GoUp = 1
			case "trGoToMinButton":
				if (strlen(trace) && strlen(csr))
					Wave work = CsrWaveRef($csr,WinName(0, 1))
					WaveStats/Q work
					if (GoUp)
						setX = WaveDims(work) == 2 ? V_maxRowLoc : V_maxLoc
						setY = V_maxColLoc
					else
						setX = WaveDims(work) == 2 ? V_minRowLoc : V_minLoc
						setY = V_minColLoc
					endif
					if (strlen(ImageInfo(WinName(0, 1),trace,0)))
						Cursor/I $csr, $trace, setX, setY									// move cursor to image position
					else
						Cursor	 $csr, $trace, setX											// move cursor to trace position
					endif
					FetchLatestCursorValues(csr)
				endif
			break
			case "trNextTraceButton":
				GoUp = 1
			case "trPrevTraceButton":
				if (strlen(trace) && strlen(csr))
					items = FetchTraces(gName,traces,0)
					setX  = hcsr($csr)
					Variable position = WhichListItem(trace, traces)
					String nextTrace = ""
					if (GoUp)
						if (position < items-1)
							 nextTrace = StringFromList(position+1,traces)
						endif
					else
						if (position > 0)
							 nextTrace = StringFromList(position-1,traces)
						endif
					endif
					if (!strlen(ImageInfo(WinName(0, 1),trace,0)) && strlen(nextTrace))
						Cursor $csr, $nextTrace, setX										// move cursor to next trace
					endif
					FetchLatestCursorValues(csr)
				endif
			break
			case "trAddXButton":
				GoUp = 1
			case "trRemXButton":
				if (strlen(trace) && strlen(csr))
					GoUp = GoUp == 0 ? -1 : GoUp
					ControlInfo/W=$(SB.win) trOffxDelta
					setX = V_Value
					if (setX < 0)
						xScale = xScale == 0 ? 1 : xScale
						ModifyGraph muloffset($trace)={-GoUp*abs(xScale),}
					else
						xScale = xScale == -1 ? 0 : xScale
						ModifyGraph muloffset($trace)={abs(xScale),}
					endif
					ModifyGraph offset($trace)={(GoUp*abs(setX)+xOff),}
					FetchLatestCursorValues(csr)
				endif
			break
			case "trAddYButton":
				GoUp = 1
			case "trRemYButton":
				if (strlen(trace) && strlen(csr))
					GoUp = GoUp == 0 ? -1 : GoUp
					ControlInfo/W=$(SB.win) trOffyDelta
					setY = V_Value
					if (setY < 0)
						yScale = yScale == 0 ? 1 : yScale
						ModifyGraph muloffset($trace)={,-GoUp*abs(yScale)}
					else
						yScale = yScale == -1 ? 0 : yScale
						ModifyGraph muloffset($trace)={,abs(yScale)}
					endif
					ModifyGraph offset($trace)={,(GoUp*abs(setY)+yOff)}
					AdjustProfileAxisScaling()
					FetchLatestCursorValues(csr)
				endif
			break
			case "trNormTo1Button":
				if (strlen(trace) && strlen(csr))
					if (vcsr($csr) != 0)
						ModifyGraph muloffset($trace)={,1/vcsr($csr)}
						FetchLatestCursorValues(csr)
					endif
				endif
			break
			case "trApplyCurButton":
				if (strlen(trace))																// falls through with only the current trace selected
					traces = trace
					items  = 1
				endif
			case "vaApplyAllButton":
				if (!strlen(traces))
					items = FetchTraces(gName,traces,0)
				endif
				for (i = 0; i < items; i += 1)
					trace = StringFromList(i,traces)
					sscanf StringByKey("offset(x)", TraceInfo(gName,trace,0), "="), "{%f,%f}", xOff, yOff
					sscanf StringByKey("muloffset(x)", TraceInfo(gName,trace,0), "="), "{%f,%f}", xScale, yScale
					yScale = yScale == 0? 1 : yScale
					xScale = xScale == 0? 1 : xScale
					
					Wave work  = TraceNameToWaveRef(gName, trace)								// apply all scales and offsets to 1D data
					Wave/Z w_x = XWaveRefFromTrace(gName, trace)
					if (WaveDims(work) != 1)													// just in case there is 2D data mixed in
						continue
					endif
					work *= yScale
					work += yOff
					if(WaveExists(w_x))
						w_x *= xScale
						w_x += xOff
					else
						SetScale/P x, DimOffset(work,0) * xScale + xOff, DimDelta(work,0) * xScale, WaveUnits(work,0), work
					endif
					ModifyGraph offset($trace)={0,0} ,muloffset($trace)={0,0}
					
					String lognote = ""															// write a simple log
					lognote += SelectString((xScale!=1),"","\rx-scale="+num2str(xScale))
					lognote += SelectString((xOff!=0),"","\rx-offset="+num2str(xOff))
					lognote += SelectString((yScale!=1),"","\ry-scale="+num2str(yScale))
					lognote += SelectString((yOff!=0),"","\ry-offset="+num2str(yOff))
					if (strlen(lognote))
						lognote[0] = "\rscaling and offsets applied:"
						Note work, lognote
					endif
				endfor
				FetchLatestTraceValues()
			break
		EndSwitch
	endif
	return 0
End

//#################################### popup helper function #####################################

Function TraceTool_ProfileSelect(s) : PopupMenuControl
	STRUCT WMPopupAction &s
	if (s.eventCode != 2)
		return 0
	endif
	SetActiveSubwindow $WinName(0,1)															// make sure to do stuff in the graph window
	String axes = AxisList(WinName(0,1))
	if (FindListItem("Profileleft", axes) != -1)												// see if the additional Profile axis is attached
		RemoveFromGraph $("AddTraceProfile")													// remove the old trace (if present)
		if (WhichListItem("bottom",axes) > -1)													// reset the axes and remove the drawn line from previous selections
			ModifyGraph axisEnab(bottom)={0,1}
		endif
		if (WhichListItem("top",axes) > -1)
			ModifyGraph axisEnab(top)={0,1}
		endif
	endif

	StrSwitch(s.popStr)
		case "none":																			// do nothing
			break
		default:
			Wave inwave = $s.popStr
			if (WhichListItem("bottom",axes) > -1)												// make the bottom axis shorter
				ModifyGraph axisEnab(bottom)={0,0.72}
			endif
			if (WhichListItem("top",axes) > -1)
				ModifyGraph axisEnab(top)={0,0.72}
			endif
			AppendToGraph/C=(0,15872,65280)/B=Profilebottom/L=Profileleft/VERT inwave/TN=AddTraceProfile	// append the Profile trace
			ModifyGraph mirror(Profilebottom)=2
			ModifyGraph axisEnab(Profilebottom)={0.76,1}	
			ModifyGraph freePos(Profilebottom)=0 ,freePos(Profileleft)={0.76,kwFraction}		// move the new axes to the right position
			ModifyGraph tkLblRot(Profileleft)=90												// rotate the tick labels
			ModifyGraph minor(Profileleft)=1, tick(Profileleft)=2								// minor ticks
			ModifyGraph standoff=0																// axes are aligned, having no 1 pixel standoff
			ModifyGraph ZisZ(Profilebottom)=1,	ZisZ(Profileleft)=1								// zero is 0 (not 0.00)
			ModifyGraph notation(Profilebottom)=1												// scientific notation (10^4 instead of 10*10^3)
			ModifyGraph tlOffset(Profileleft)=-2												// move the labels in a bit
			AdjustProfileAxisScaling()															// resizes the profile axis
		break
	EndSwitch
	return 0
End

//################################################################################################

Function ColorSpectrumFromTable(rev, table)
	Variable rev
	String table
	
	String traces = ""
	Variable i, items = FetchTraces(WinName(0,1),traces,0)
	if (items == 0)																				// may be an image
		traces = StringFromList(0,ImageNameList("",";"))
		if (strlen(traces) == 0)
			return 0
		endif
		ModifyImage $traces ctab={,,$table,rev}
		return 0
	endif
	
	ColorTab2Wave $table
	Wave M_colors
	for (i = 0; i < items; i += 1)
		Variable row = (i/(items-1))*(DimSize(M_Colors, 0)-1)
		Variable red = M_Colors[row][0], green = M_Colors[row][1], blue = M_Colors[row][2]
		Variable idx = rev == 0 ? i : (items-1)-i
		ModifyGraph/Z rgb($StringFromList(idx,traces)) = (red, green, blue)
	endfor
	KillWaves/Z M_colors
	return 0
End

//________________________________________________________________________________________________
//
//										GRAPH SIZE PANEL
//________________________________________________________________________________________________

Function BuildGraphSizePanel()
	if (WinType("GraphSizePanel"))
		DoWindow/F GraphSizePanel
		return 0
	endif
	NewPanel/K=1/W=(500,60,820,415)/N=GraphSizePanel as "Set Graph Size"
	ModifyPanel /W=GraphSizePanel ,fixedSize= 1 ,noEdit=1
	SetWindow GraphSizePanel hook(UpdateHook)=SizePanelUpdateFunc
	
	SetDrawLayer UserBack
	DrawLine 10,230,310,230
	DrawLine 10,290,310,290
	SetDrawEnv linethick= 2,fillpat= 0
	DrawRect 25,10,292,195
	SetDrawEnv linethick= 2,dash= 3,fillpat= 0
	DrawRect 55,40,260,165
	SetDrawEnv linethick= 2,linefgc= (0,0,65535),arrow= 3
	DrawLine 58,70,257,70
	SetDrawEnv linethick= 2,linefgc= (0,0,65535),arrow= 3
	DrawLine 210,43,210,163
	SetDrawEnv linethick= 2,linefgc= (1,39321,19939),arrow= 3
	DrawLine 33,135,285,135
	SetDrawEnv linethick= 2,linefgc= (1,39321,19939),arrow= 3
	DrawLine 110,13,110,193
	
	SetVariable VLeftMargin		,pos={10,95}	,value=_NUM:0
	SetVariable VBottomMargin	,pos={133,170}	,value=_NUM:0
	SetVariable VRightMargin	,pos={250,95}	,value=_NUM:0
	SetVariable VTopMargin		,pos={132,15}	,value=_NUM:0
	SetVariable VGraphHeight	,pos={80,95}	,value=_NUM:0
	SetVariable VPlotWidth		,pos={130,60}	,value=_NUM:0
	SetVariable VGraphWidth		,pos={130,125}	,value=_NUM:0
	SetVariable VPlotHeight		,pos={180,95}	,value=_NUM:0
	SetVariable VMagnification	,pos={220,170}	,value=_NUM:0
	SetVariable VGraphFont		,pos={42,170}	,value=_NUM:0
	
	ModifyControlList ControlNameList("GraphSizePanel",";","*")	,limits={0,2000,1}	,size={60.00,18.00} ,bodyWidth=60 ,proc=Gsize_VarControl
	SetVariable VMagnification	,title="exp:"	,limits={0,8,0} 	,bodyWidth=40	,format="%g x"
	SetVariable VGraphFont		,title="font:"	,limits={0,100,0}	,bodyWidth=40
	
	TitleBox SwapTitle			,pos={145,80}	,size={60,23}	,title="\JCswap:"			,frame=0
	PopupMenu SizeUnit			,pos={195,15}	,size={70,23}	,title="unit: "				,mode=1		,proc=Gsize_PresetPopup		,popvalue="points"	,value= #"\"points;inches;cm\""
	CheckBox SwapAxes			,pos={152,98}	,size={23,23}	,title=""					,value=0	,proc=Gsize_SwapGraph
	
	TitleBox AxisCheckTitle		,pos={15,207}	,size={60,23}	,title="Ticks & Labels:"	,frame=0
	CheckBox cAxis_left			,pos={100,207}	,size={23,23}	,title="Left"				,value=0
	CheckBox cAxis_bottom		,pos={145,207}	,size={23,23}	,title="Bottom"				,value=0
	CheckBox cAxis_right		,pos={210,207}	,size={23,23}	,title="Right"				,value=0
	CheckBox cAxis_top			,pos={265,207}	,size={23,23}	,title="Top"				,value=0
	
	CheckBox KeepAutoMode		,pos={10,267}	,size={23,23}	,title="Keep Auto Mode"		,value=1	,help={"Automatically resets the mode to Auto after changing the size."}
	PopupMenu AspectSelect		,pos={10,237}	,size={145,22}	,title="Aspect Ratio:"	,bodyWidth=70	,proc=Gsize_PresetPopup
	PopupMenu FormatSelect		,pos={135,237}	,size={175,22}	,title="Format:"		,bodyWidth=100	,proc=Gsize_PresetPopup
	Button CopyAllButton		,pos={125,263}	,size={85,22}	,title="Copy Values"
	Button PasteAllButton		,pos={225,263}	,size={85,22}	,title="Paste Copied"

	Button CopyStyleButton		,pos={10,297}	,size={85,22}	,title="Copy Style"						,help={"Copies the style of the top graph."}
	CheckBox CopyAxes			,pos={110,300}	,size={60,22}	,title="w/ Axis Scaling"	,value=1	,help={"Copies the axis scaling as well."}
	PopupMenu GrStyle			,pos={10,327}	,size={200,22}	,title="Style:"			,bodyWidth=165
	Button RevertButton			,pos={225,297}	,size={85,22}	,title="Revert"							,help={"Reverts to last style before Apply was pressed."}
	Button ApplyButton			,pos={225,325}	,size={85,22}	,title="\f01Apply"						,help={"Applies selected style to the top graph."}
	
	PopupMenu GrStyle			,mode=1	,value= Gsize_FetchGraphMacroList()
	PopupMenu AspectSelect		,mode=1	,value= "Free;1:1;-;4:3;3:2;16:9;-;2:3;3:4;4:5"
	PopupMenu FormatSelect		,mode=1	,value= "Free Size;Paper Half;Paper Full;PPT 16:9 Half;PPT 16:9 Full;PPT 4:3 Half;PPT 4:3 Full;"
	
	ModifyControlList ControlNameList("GraphSizePanel",";","*Button")	,proc=Gsize_CopyPaste
	ModifyControlList ControlNameList("GraphSizePanel",";","cAxis_*")	,proc=Gsize_ToggleAxis
	
	if (CmpStr(IgorInfo(2), "Macintosh") == 0)
		ModifyControlList ControlNameList("OffsetPanel",";","*") ,fsize=10
	endif
	FetchLatestGraphSizeValues()
	return 0
End
//++++++++++++++++++++++++++++++++ panel update helper function ++++++++++++++++++++++++++++++++++
Function SizePanelUpdateFunc(s)
	STRUCT WMWinHookStruct &s
	If (s.eventCode == 0)
		FetchLatestGraphSizeValues()
		return 1
	endif
	return 0
End

//################################################################################################

Function/S Gsize_FetchGraphMacroList()
	String list = MacroList("*",";","KIND:1,SUBTYPE:GraphStyle")
	String copy = GetUserData("GraphSizePanel", "", "CopyStyle")
	if (strlen(copy))
		list[0] = "Copied Style;"
	endif
	return list
End

//################################################################################################

Function Gsize_SwapGraph(cb) : CheckBoxControl
	STRUCT WMCheckboxAction& cb
	if (cb.eventCode == 2 && strlen(WinName(0,1)) > 0)
		ControlInfo/W=GraphSizePanel AspectSelect
		String currAspect = S_Value
		ChangeAspectRatio("Free")																// release the aspect mode and set again later (otherwise swap will switch this too)
		ModifyGraph/Z swapXY=cb.checked
		ChangeAspectRatio(currAspect)
		FetchLatestGraphSizeValues()
	endif
	return 0
End

//################################################################################################

Function Gsize_ToggleAxis(cb) : CheckBoxControl
	STRUCT WMCheckboxAction& cb
	if (cb.eventCode == 2 && strlen(WinName(0,1)) > 0)
		SetAxisLabelsAndTicks(ReplaceString("cAxis_",cb.ctrlName,""), WinName(0,1), !cb.checked)
		FetchLatestGraphSizeValues()
	endif
	return 0
End
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Static Function SetAxisLabelsAndTicks(axName, gName, off)
	String axName, gName
	Variable off
	if (FindListItem(axName,AxisList(gName)) > -1)
		ModifyGraph/W=$gName tick($axName)=3*off
		ModifyGraph/W=$gName noLabel($axName)=2*off
		ModifyGraph/W=$gName margin($axName)=25*off
	endif
	return 0
End

//################################################################################################

Function Gsize_CopyPaste(s) : ButtonControl
	STRUCT WMButtonAction &s
	if (s.eventCode != 2)
		return 0
	endif
	String gName = WinName(0,1)
	if (!strlen(gName))
		return 0
	endif

	String CtrlList = "", ValList = "", Others = "", AxisSet = "", Style = ""
	Variable i
	StrSwitch(s.ctrlName)
		case "CopyAllButton":
			CtrlList = ControlNameList("GraphSizePanel", ";", "cAxis_*")
			for (i = 0; i < ItemsInList(CtrlList); i += 1)
				ControlInfo/W=GraphSizePanel $StringFromList(i,CtrlList)
				AxisSet += num2str(V_Value) + ";"
			endfor

			CtrlList = ControlNameList("GraphSizePanel", ";", "V*")
			CtrlList = RemoveFromList("VGraphWidth;VGraphHeight;",CtrlList)
			for (i = 0; i < ItemsInList(CtrlList); i += 1)
				ControlInfo/W=GraphSizePanel $StringFromList(i,CtrlList)
				ValList += num2str(V_Value) + ";"
			endfor
			ControlInfo/W=GraphSizePanel SwapAxes
			Others +=  "SwapAxes:" + num2str(V_Value) + ";"
			ControlInfo/W=GraphSizePanel SizeUnit
			Others +=  "SizeUnit:" + S_Value + ";"
			ControlInfo/W=GraphSizePanel AspectSelect
			Others +=  "Aspect:" + S_Value + ";"
			ControlInfo/W=GraphSizePanel FormatSelect
			Others +=  "Format:" + S_Value + ";"
		
			SetWindow GraphSizePanel ,userdata(CtrlSet)=CtrlList								// saved in the panel's user data
			SetWindow GraphSizePanel ,userdata(ValSet)=ValList
			SetWindow GraphSizePanel ,userdata(OtherSet)=Others
			SetWindow GraphSizePanel ,userdata(AxesSet)=AxisSet
		break
		case "PasteAllButton":
			CtrlList = GetUserData("GraphSizePanel", "", "CtrlSet")
			ValList	 = GetUserData("GraphSizePanel", "", "ValSet")
			Others   = GetUserData("GraphSizePanel", "", "OtherSet")
			AxisSet	 = GetUserData("GraphSizePanel", "", "AxesSet")
			if (strlen(CtrlList) == 0)
				break
			endif
			Variable swap = str2num(StringByKey("SwapAxes", Others))
			CheckBox SwapAxes  		,win=GraphSizePanel	,value=swap
			PopupMenu SizeUnit 		,win=GraphSizePanel	,popMatch=StringByKey("SizeUnit", Others)
			PopupMenu FormatSelect	,win=GraphSizePanel	,popMatch=StringByKey("Format", Others)
			PopupMenu AspectSelect	,win=GraphSizePanel	,popMatch=StringByKey("Aspect", Others)
			
			String cName
			Variable cValue
			for (i = 0; i < ItemsInList(CtrlList); i += 1)
				cName  = StringFromList(i,CtrlList)
				cValue = str2num(StringFromList(i,ValList))
				SetVariable $cName	,win=GraphSizePanel ,value=_NUM:cValue
				ExecuteVarChange(cName, cValue)
			endfor
			
			ModifyGraph/Z swapXY=swap
			String aspect = StringByKey("Aspect", Others)
			if (!StringMatch(aspect,"Free"))													// actually apply aspect ratio setting as well (should come after swapXY)
				ChangeAspectRatio(aspect)
			endif

			CtrlList = ControlNameList("GraphSizePanel", ";", "cAxis_*")
			for (i = 0; i < ItemsInList(CtrlList); i += 1)
				cName  = ReplaceString("cAxis_",StringFromList(i,CtrlList),"")
				cValue = str2num(StringFromList(i,AxisSet)) == 0
				SetAxisLabelsAndTicks(cName, WinName(0,1), cValue)
			endfor
		break
		case "ApplyButton":																		// applies selected graph style
			ControlInfo/W=$(s.win) GrStyle
			Style = S_value
			String Last = WinRecreation(gName,0)
			if (StringMatch(Style,"Copied Style"))
				Style = GetUserData(s.win, "", "CopyStyle") 
				for (i = 0; i < ItemsInList(Style); i += 1)
					Execute/Q StringFromList(i,Style)
				endfor
			elseif (strlen(Style))
				Execute/Q Style+"()"
			endif
			SetWindow $(s.win), userdata(LastgName)=gName										// save into panel's user data
			SetWindow $(s.win), userdata(LastStyle)=Last
		break
		case "RevertButton":																	// copies top-graph's style
			Style = GetUserData(s.win, "", "LastStyle")											// check for existence of backup style
			if (strlen(Style) > 0)
				KillWindow $gName
				String/G ExeCmd = Style
				Execute/P/Q "Execute ExeCmd"
				Execute/P/Q "KillStrings/Z ExeCmd"
			endif
		break
		case "CopyStyleButton":																	// copies top-graph's style
			Style = WinRecreation(gName,1)
			Style = GrepList(Style, "^\t(DefineGuide|ControlBar|SetWindow|Cursor|ListBox|CheckBox|PopupMenu|ValDisplay|SetVariable|Button|NewPanel|ModifyPanel|RenameWindow|SetActiveSubwindow|ShowTools)", 1, "\r") // remove control items
			Style = GrepList(Style, "^\t(ModifyGraph/Z rgb|ModifyGraph/Z muloffset|ModifyGraph/Z offset|ShowInfo)", 1, "\r")
			ControlInfo/W=$(s.win) CopyAxes														// include axes if checkbox is active
			if (!V_Value)
				Style = GrepList(Style, "^\t(SetAxis)", 1, "\r")
			endif
			Style = RemoveListItem(0,Style, "\r")												// remove "proc" line
			Style = RemoveListItem(0,Style, "\r")												// remove "pauseupdate" line
			Style = RemoveListItem(ItemsInList(Style,"\r") - 1,Style, "\r")						// remove "endmacro" line
			Style = ReplaceString("\t", Style, "")
			Style = ReplaceString("\r", Style, ";")
			SetWindow $s.win, userdata(CopyStyle)=Style											// save into panel's user data
			PopupMenu GrStyle ,win=$s.win ,mode= 1
		break
	EndSwitch
	FetchLatestGraphSizeValues()
	return 0
End

//################################################################################################

Function Gsize_VarControl(s) : SetVariableControl
	struct WMSetVariableAction &s
	if (s.eventCode == 1 || s.eventCode == 2)
		ExecuteVarChange(s.ctrlName, s.dval)
	endif
	return 0
End
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Static Function ExecuteVarChange(ctrlName, val)
	String ctrlName
	Variable val
	String gName = WinName(0,1)
	if (!strlen(gName))
		return 0
	endif

	ControlInfo/W=GraphSizePanel SizeUnit
	Variable modifier = V_Value == 1 ? 1 : (V_Value == 2 ? 72 : 72/2.54)						// in points, inches or cm
	ControlInfo/W=GraphSizePanel KeepAutoMode
	Variable Auto = V_Value																		// reset to Auto scale mode after size change
	
	StrSwitch(ctrlName)
		case "VLeftMargin":
			ModifyGraph/W=$gName margin(left) = val*modifier
		break
		case "VBottomMargin":
			ModifyGraph/W=$gName margin(bottom) = val*modifier
		break
		case "VRightMargin":
			ModifyGraph/W=$gName margin(right) = val*modifier
		break
		case "VTopMargin":
			ModifyGraph/W=$gName margin(top) = val*modifier
		break
		case "VGraphHeight":
			ControlInfo/W=GraphSizePanel VTopMargin
			Variable mTop = V_Value
			ControlInfo/W=GraphSizePanel VBottomMargin
			Variable mBot = V_Value
			Variable pHeight = val-mTop-mBot
			if (val == 0)
				ModifyGraph/W=$gName margin(top) = 0, margin(bottom) = 0,  height = 0
			elseif (pHeight > 0)
				ModifyGraph/W=$gName height = pHeight*modifier
			endif
		break
		case "VGraphWidth":
			ControlInfo/W=GraphSizePanel VLeftMargin
			Variable mLeft = V_Value
			ControlInfo/W=GraphSizePanel VRightMargin
			Variable mRight = V_Value
			Variable pWidth = val-mLeft-mRight
			if (val == 0)
				ModifyGraph/W=$gName margin(left) = 0, margin(right) = 0,  width = 0
			elseif (pWidth > 0)
				ModifyGraph/W=$gName width = pWidth*modifier
			endif
		break
		case "VPlotHeight":
			ModifyGraph/W=$gName height = val*modifier
		break
		case "VPlotWidth":
			ModifyGraph/W=$gName width = val*modifier
		break
		case "VMagnification":
			Variable Mag = val == 1 ? 0 : val	// reset to none
			ModifyGraph/W=$gName expand = Mag
		break
		case "VGraphFont":
			ModifyGraph/W=$gName gfSize = val
		break
	EndSwitch
	
	if (Auto)
		DoUpdate/W=$gName
		if (StringMatch(ctrlName,"*Height"))
			ModifyGraph/W=$gName height = 0
		elseif (StringMatch(ctrlName,"*Width"))
			ModifyGraph/W=$gName width = 0
		endif
	endif
	FetchLatestGraphSizeValues()
	return 0
end

//################################################################################################

Function Gsize_PresetPopup(s) : PopupMenuControl
	STRUCT WMPopupAction &s
	if (s.eventCode == 2)
		StrSwitch(s.ctrlName)
			case "AspectSelect":
				ChangeAspectRatio(s.popStr)
			break
			case "FormatSelect":
				ChangeGraphFormat(s.popStr)
			break
			case "SizeUnit":
				String CtrlList = RemoveFromList("VMagnification;VGraphFont;",ControlNameList("GraphSizePanel",";","V*"))
				Variable div  = s.popNum == 1 ? 1 : (s.popNum == 2 ? 72 : 72/2.54)				// in points, inches or cm
				Variable step = s.popNum == 1 ? 1 : (s.popNum == 2 ? 0.014 : 0.036)
				ModifyControlList CtrlList ,win=GraphSizePanel	,limits={0,8000/div,step}		// official max size is 8000 points
				FetchLatestGraphSizeValues()
			break
		EndSwitch
	endif
	return 0
End
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Static Function ChangeAspectRatio(which)
	String which
	String gName = WinName(0,1)
	if (!strlen(gName))
		return 0
	endif
	String HeightMode = ""
	StrSwitch (which)
		case "Free":
			HeightMode = "0"
		break
		case "1:1":
			HeightMode = "{Aspect,1}"
		break
		case "4:3":
			HeightMode = "{Aspect,3/4}"
		break
		case "3:2":
			HeightMode = "{Aspect,2/3}"
		break
		case "16:9":
			HeightMode = "{Aspect,9/16}"
		break
		case "2:3":
			HeightMode = "{Aspect,3/2}"
		break
		case "3:4":
			HeightMode = "{Aspect,4/3}"
		break
		case "4:5":
			HeightMode = "{Aspect,5/4}"
		break
	EndSwitch
	if (strlen(HeightMode) > 0)
		Execute/Q "ModifyGraph/Z height="+HeightMode+";FetchLatestGraphSizeValues();"
	endif
	return 0
End
// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Static Function ChangeGraphFormat(which)
	String which
	String gName = WinName(0,1)
	if (!strlen(gName))
		return 0
	endif
	PopupMenu AspectSelect	,win=GraphSizePanel	,popMatch="Free"								// reset aspect popup
	if (StringMatch(which,"Free Size"))
		ModifyGraph/Z width=0, height=0, margin=0
		ModifyGraph/Z gfSize=0, fSize=0, lsize=1
		ModifyGraph/Z expand=0
		FetchLatestGraphSizeValues()
		return 0
	endif
	
	ControlInfo/W=GraphSizePanel KeepAutoMode
	Variable Auto = V_Value
	GetAxis/Q bottom																			// check which axes are active (increases the margin here)
	Variable bottomOn = !V_Flag
	GetAxis/Q left
	Variable leftOn = !V_Flag
	GetAxis/Q right
	Variable rightOn = !V_Flag
	GetAxis/Q top
	Variable topOn = !V_Flag
	
	if (StringMatch(which, "Paper*")) 
		ModifyGraph/Z gfSize=8, fSize=8
		ModifyGraph/Z expand=1.25
		ModifyGraph/Z lsize=1.5
		
		ModifyGraph/Z margin(left)	= (11 + 22*leftOn + 10*(bottomOn || topOn && !leftOn))
		ModifyGraph/Z margin(bottom)= (11 + 22*bottomOn)
		ModifyGraph/Z margin(top)	= (11 + 22*topOn)
		ModifyGraph/Z margin(right)	= (11 + 22*rightOn + 10*(bottomOn || topOn && !rightOn))
		if (StringMatch(which, "*Half"))
			ModifyGraph/Z width=(180 - 12*(leftOn && rightOn))
			ModifyGraph/Z height=(172 - 22*(bottomOn && topOn))
		elseif (StringMatch(which, "*Full"))
			ModifyGraph/Z width=(450 - 12*(leftOn && rightOn))
			ModifyGraph/Z height=(208 - 22*(bottomOn && topOn))
		endif
	endif
	
	if (StringMatch(which, "PPT*")) 
		ModifyGraph/Z gfSize=16, fSize=12
		ModifyGraph/Z expand=0.75
		ModifyGraph/Z lsize=1.5

		ModifyGraph/Z margin(left)	= (18 + 37*leftOn + 12*(bottomOn || topOn && !leftOn))
		ModifyGraph/Z margin(bottom)= (18 + 37*bottomOn)
		ModifyGraph/Z margin(top)	= (18 + 37*topOn)
		ModifyGraph/Z margin(right)	= (18 + 37*rightOn + 12*(bottomOn || topOn && !rightOn))
		if (StringMatch(which, "*16:9*"))
			ModifyGraph/Z height=(360 - 37*(bottomOn && topOn))
			if (StringMatch(which, "*Half"))
				ModifyGraph/Z width=(380 - 25*(leftOn && rightOn))
			elseif (StringMatch(which, "*Full"))
				ModifyGraph/Z width=(800 - 25*(leftOn && rightOn))
			endif
		elseif (StringMatch(which, "*4:3*"))
			ModifyGraph/Z height=(350 - 37*(bottomOn && topOn))
			if (StringMatch(which, "*Half"))
				ModifyGraph/Z width=(290 - 25*(leftOn && rightOn))
			elseif (StringMatch(which, "*Full"))
				ModifyGraph/Z width=(600 - 25*(leftOn && rightOn))
			endif
		endif
	endif

	DoUpdate
	if (Auto)
		ModifyGraph/Z height = 0
		ModifyGraph/Z width = 0
	endif
	FetchLatestGraphSizeValues()
	return 0
End

//################################################################################################

Function FetchLatestGraphSizeValues()
	String gName = WinName(0,1), gRec = ""
	if (!strlen(gName))
		DoWindow /T GraphSizePanel, "Set Graph Size: No Graph"
		return 0
	endif
	
	DoUpdate/W=$gName
	GetWindow $gName gsize																		// values from GetWindow (sometimes off by a fraction?)
	//print V_left, V_right, V_top, V_bottom		// debug
  	Variable gLeft	= V_left
  	Variable gTop	= V_top
  	Variable gWidth	= V_right-V_left	
  	Variable gHeight= V_bottom-V_top	
	GetWindow $gName psize
	//print V_left, V_right, V_top, V_bottom		// debug
	Variable mLeft	= round(V_left - gLeft)
	Variable mTop	= round(V_top - gTop)
	Variable mRight	= round(gWidth - V_right)
	Variable mBottom= round(gHeight - V_bottom)
	Variable pWidth	= round(V_right - V_left)
	Variable pHeight= round(V_bottom - V_top)
	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	gRec = GrepList(WinRecreation(gName, 0), "^\t(ModifyGraph)", 0 , "\r")						// if available, replace with values from recreation macro (these are exact)
	gRec = GrepList(gRec, "(?i)(height|width|margin|gfsize|swapXY)", 0 , "\r")					// extract only relevant keys
	gRec = ReplaceString("\tModifyGraph ",gRec,"")
	gRec = ReplaceString("\r",gRec,",")

	Variable gSwap = str2num(StringByKey("swapXY", gRec, "=", ","))								// swap axes parameter
	Variable gMagn = abs(str2num(StringByKey("expand", gRec, "=", ",")))						// magnification parameter
	gSwap = numtype(gSwap) == 0 ? gSwap : 0
	gMagn = numtype(gMagn) == 0 ? gMagn : 1
	Variable Rec_mLeft	 = gSwap ? str2num(StringByKey("margin(bottom)", gRec, "=", ",")): str2num(StringByKey("margin(left)", gRec, "=", ","))
	Variable Rec_mTop	 = gSwap ? str2num(StringByKey("margin(right)", gRec, "=", ",")) : str2num(StringByKey("margin(top)", gRec, "=", ","))
	Variable Rec_mRight	 = gSwap ? str2num(StringByKey("margin(top)", gRec, "=", ","))	 : str2num(StringByKey("margin(right)", gRec, "=", ","))
	Variable Rec_mBottom = gSwap ? str2num(StringByKey("margin(left)", gRec, "=", ","))	 : str2num(StringByKey("margin(bottom)", gRec, "=", ","))
	Variable Rec_pWidth	 = gSwap ? str2num(StringByKey("height", gRec, "=", ","))		 : str2num(StringByKey("width", gRec, "=", ","))
	Variable Rec_pHeight = gSwap ? str2num(StringByKey("width", gRec, "=", ","))		 : str2num(StringByKey("height", gRec, "=", ","))
	Variable Rec_gHeight = Rec_pHeight + Rec_mBottom + Rec_mTop
	Variable Rec_gWidth	 = Rec_pWidth  + Rec_mRight  + Rec_mLeft
	Variable Rec_gFont	 = str2num(StringByKey("gfSize", gRec, "=", ","))
	Variable gFont = numtype(Rec_gFont) == 0 ? Rec_gFont : 10
#if IgorVersion() >= 7.00
	GetWindow $gName expand																		// fetch magnification in newer Igor versions
	gMagn = V_Value == 0 ? 1 : V_Value
#endif
	// ++++++++++++++++++++++++++++++++ get aspect ratio info ++++++++++++++++++++++++++++++++++++
	gRec = StringFromList(1, ReplaceString("{", ReplaceString("}",StringByKey(SelectString(gSwap, "height", "width"), gRec, "="),""),""), ",")	// extract aspect ratio
	String PopList = "free;1:1;4:3;3:2;16:9;2:3;3:4;4:5"										// popup match list
	Variable sel = 0																			// extract aspect ratio
	sel += 1 * StringMatch(gRec,"1")
	sel += 2 * StringMatch(gRec,"0.75")
	sel += 3 * StringMatch(gRec,"0.666667")
	sel += 4 * StringMatch(gRec,"0.5625")
	sel += 5 * StringMatch(gRec,"1.5")
	sel += 6 * StringMatch(gRec,"1.33333")
	sel += 7 * StringMatch(gRec,"1.25")
	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	mLeft	= numtype(Rec_mLeft) == 0 ? Rec_mLeft : mLeft										// if the recreation values are valid then use these
	mBottom	= numtype(Rec_mBottom) == 0 ? Rec_mBottom : mBottom
	mRight	= numtype(Rec_mRight) == 0 ? Rec_mRight : mRight
	mTop	= numtype(Rec_mTop) == 0 ? Rec_mTop : mTop
	pWidth	= numtype(Rec_pWidth) == 0 ? Rec_pWidth : pWidth
	pHeight	= numtype(Rec_pHeight) == 0 ? Rec_pHeight : pHeight
	gHeight	= numtype(Rec_gHeight) == 0 ? Rec_gHeight : pHeight + mTop + mBottom
	gWidth	= numtype(Rec_gWidth) == 0 ? Rec_gWidth : pWidth + mLeft + mRight
	
	ControlInfo/W=GraphSizePanel SizeUnit
	Variable modifier = V_Value == 1 ? 1 : (V_Value == 2 ? 72 : 72/2.54)						// in points, inches or cm
	if (modifier != 1)
		mLeft	= round((mLeft	 /modifier)*1000)/1000
		mBottom	= round((mBottom /modifier)*1000)/1000
		mRight	= round((mRight	 /modifier)*1000)/1000
		mTop	= round((mTop	 /modifier)*1000)/1000
		gHeight	= round((gHeight /modifier)*1000)/1000
		pWidth	= round((pWidth	 /modifier)*1000)/1000
		gWidth	= round((gWidth	 /modifier)*1000)/1000
		pHeight	= round((pHeight /modifier)*1000)/1000
	endif
	// +++++++++++++++++++++++++++++++ apply info to panel controls ++++++++++++++++++++++++++++++
	String undoGName = GetUserData("GraphSizePanel", "", "LastgName")							// check for existence of a undo style for the current graph
	DoWindow /T GraphSizePanel, "Set Graph Size: "+WinName(0,1)
	CheckBox SwapAxes			,win=GraphSizePanel	,value=gSwap
	SetVariable VLeftMargin		,win=GraphSizePanel	,value=_NUM:mLeft	,fstyle=(numtype(Rec_mLeft)==0)
	SetVariable VBottomMargin	,win=GraphSizePanel	,value=_NUM:mBottom	,fstyle=(numtype(Rec_mBottom)==0)
	SetVariable VRightMargin	,win=GraphSizePanel	,value=_NUM:mRight	,fstyle=(numtype(Rec_mRight)==0)
	SetVariable VTopMargin		,win=GraphSizePanel	,value=_NUM:mTop	,fstyle=(numtype(Rec_mTop)==0)
	SetVariable VGraphHeight	,win=GraphSizePanel	,value=_NUM:gHeight	,fstyle=(numtype(Rec_gHeight)==0)
	SetVariable VPlotWidth		,win=GraphSizePanel	,value=_NUM:pWidth	,fstyle=(numtype(Rec_pWidth)==0)
	SetVariable VGraphWidth		,win=GraphSizePanel	,value=_NUM:gWidth	,fstyle=(numtype(Rec_gWidth)==0)
	SetVariable VPlotHeight		,win=GraphSizePanel	,value=_NUM:pHeight	,fstyle=(numtype(Rec_pHeight)==0)
	SetVariable VGraphFont		,win=GraphSizePanel	,value=_NUM:gFont	,fstyle=(numtype(Rec_gFont)==0)
	SetVariable VMagnification	,win=GraphSizePanel	,value=_NUM:gMagn
	PopupMenu AspectSelect		,win=GraphSizePanel	,popMatch=StringFromList(sel,PopList)
	Button RevertButton			,win=GraphSizePanel	,disable=2*(!StringMatch(gName,undoGName))	// disable button if undo backup does not match to current graph
	
	Variable i
	String axes = "left;bottom;right;top;"
	for (i = 0; i < ItemsInList(axes); i += 1)													// set axis checkboxes
		Variable isOn = 0
		String currAxis = StringFromList(i,axes)
		gRec = AxisInfo(gName,currAxis)
		if (strlen(gRec))
			isOn = (str2num(StringByKey("tick(x)",gRec,"=",";"))!=3 && str2num(StringByKey("noLabel(x)",gRec,"=",";"))!= 2)		// inactive if both tick(x)=3 and noLabel(x)=2
		endif
		CheckBox $("cAxis_"+currAxis) ,win=GraphSizePanel ,value=isOn ,disable=2*(strlen(gRec)==0)
	endfor
	return 0
End